% Sesion 7 derivacin e integracin numrica

%1. Derivacin numrica: diferencias finitas centradas, regresivas y progresivas

%f ' (a) ~ ( f (a+h) - f (a-h) )/ (2 h)
%f ' (a) ~ ( f (a) - f (a-h) )/ (h)
%f ' (a) ~ ( f (a+h) - f (a) )/ (h)

%El comando de octave ms directamente relacionado con estas frmulas
%cuando y es un vector que representa las sucesivas evaluaciones de una funcin, es 
diff(y)
%proporciona un vector de diferencias 
%y(2:n)-y(1:n-1) siendo n=length(y)

%En este sentido, si x representa el vector de abscisas e y el vector 
%de las evaluaciones de cierta funcin sobre dichas abscisas y(n)=f(x(n)),
%entonces la aproximacin por diferencias progresivas de f ' (x(n))
%viene dada por %der=diff(y)./diff(x). OJO, que as aproximamos la derivada 
%en todos los puntos salvo el ltimo. Lo mostramos con la funcin seno.

x=0:0.1:pi;
y=sin(x);
der=diff(y)./diff(x);
plot(x,y,x(1:end-1),der)



%2. Ejercicio: crear una funcion que "derive numricamente" una funcin f real 
%de variable real, , es decir, que tenga a  f por entrada, un intervalo [a,b] en el 
%que queramos derivarla, y el nmero de  nodos N, produzca como salida sendos 
%vectores [X, Df] de longitud N de modo que 
%- el primero, X, sea la particicin del intervalo (para poder tratar
% despues los datos de la funcin y su derivada sobre el mismo vector de abscisas) 
%- y el segund, Df,  los valores de la derivada calculada numricamente mediante
% diferencias centradas.


function [ x, df ] = derinum ( f , a , b , N)
   h=(b-a)./(N-1);
   x=linspace (a,b,N); %ver diferencia (posible) con x = a+h(0:(N-1))
   df = (feval(f,a+h*(1:N))-feval(f,a+h*(-1:(N-2))))/(2*h);
endfunction


%comprobacion sobre ejemplos



[x,df]=derinum('sin',0,2*pi,40);
plot(x,sin(x),';funcion;',x,df,';derivada;')

[x,df]=derinum('abs',-1,1,60);
plot(x,abs(x),';funcion;',x,df,';derivada;')

function y = f (x) y=sin(1./x) ; endfunction
[x,df]=derinum('f',0.01,1,100);
plotyy (x,f(x),x,df)
plot(x,f(x),';funcion;',x,df,';derivada;')



%3. Ojo, que las formulas de derivacin son inestables.
%Veamos que al aproximar la derivada de la funcin arcotangente
%en el punto raz de 2, (cuyo valor es 1/3), por diferencias progresivas
%conforme h tiende a cero, al principio parece converger, pero a 
%partir de un momento da 0

pot=-1:(-1):(-20);
h=10.^pot;
a=sqrt(2);
%mostramos en la primera columna los valores de h y en la segunda
%los cocientes que aproximan la derivada: (atan(a+h)-atan(a))./h
format long
[h;((atan(a+h)-atan(a))./h)]'






%4. Integracin

%vamos a usar una funcin y = 2 sqrt(1-x^2) cuya integral
%entre -1 y 1 es igual a pi, para contrastar resultados.
f = @(x) 2*sqrt(1-x.^2);
x=(-1):0.05:1; 
plot(x,f(x));
hold on


%comando directo para calcular integrales quad:
%[v, ier, nfun, err ] = quad (f, a, b, tol, sing ) 
[v,ier,nfun,err] = quad(f,-1,1)

%- las entradas:
%f es la funcion a integrar, tiene que ser real de variable real
%a y b son los lmites de integracin, nmeros reales, que pueden ser infinitos
%tol es opcional, es un vector con dos componentes, la tolerancia absoluta y la 
%relativa respectivamente
%si uno de los dos se pone cero, se obvia y usa el otro
%sing es opcional, es un vector con los valores donde sepamos que f es singular
%
%-las salidas
%v es el resultado de la integral
%ier es un cdigo de error (0 es ptimo)
%nfun indica cuantas evaluaciones ha usado
%err es una estimacin del error absoluto
%en el ejemplo, ha usado 399 evaluaciones

%5. Frmula de los rectngulos (usando el punto medio en cada intervalillo)


x1=(-1):0.2:1; 
y=feval(f, x1);
bar(x1,y) 
title('rectangulos, punto medio')
hold off

%
function int = rectang (f, a, b, n = 100) %IMPORTANTE valor por defecto de n
h = (b - a) ./ n; 
int = h.*sum(feval(f, a + h * (1:n) - h/2));  %evaluaciones de f en los centros de los rectngulos
endfunction


rectang('f',-1,1)

rectang('f',-1,1,399)

%6 Ejercicio: Frmula de los trapecios, en este caso, poniendo como entradas las abscisas
%y las ordenadas (no la funcin)

function int  = trape (x,y)
%trapecion para funciones (dadas sus evaluaciones)
n=length(x);
int=sum((y(2:n)+y(1:(n-1))).*(x(2:n)-x(1:(n-1))))/2;
endfunction

n=100;
x=linspace(-1,1,n);
trape (x,f(x))

n=400;
x=linspace(-1,1,n);
trape (x,f(x))



%7. Ejercicio: Simpson, usando f.

function int = simpson (f, a, b, N = 200) 
h = (b - a) / N; 
x = a + h*(0:(N-1)); 
int = (sum(feval(f,x) + 4 * feval(f,x+h/2) + feval(f,x+h)))*h/6;
endfunction

simpson('f',-1,1)